{-# OPTIONS_GHC -Wall #-}
{-# OPTIONS_GHC -fno-warn-unused-imports #-}
{-# OPTIONS_GHC -fno-warn-orphans #-}
{-# Language ForeignFunctionInterface #-}
{-# Language FlexibleInstances #-}
{-# Language MultiParamTypeClasses #-}

module Casadi.Core.Classes.LinearSolver
       (
         LinearSolver,
         LinearSolverClass(..),
         linearSolver__0,
         linearSolver__1,
         linearSolver_prepare,
         linearSolver_prepared,
         linearSolver_solve__0,
         linearSolver_solve__1,
         linearSolver_solve__2,
         linearSolver_solve__3,
       ) where


import Prelude hiding ( Functor )

import Data.Vector ( Vector )
import Foreign.C.Types
import Foreign.Marshal ( new, free )
import Foreign.Storable ( peek )
import Foreign.Ptr ( Ptr, nullPtr )
import Foreign.ForeignPtr ( newForeignPtr )
import System.IO.Unsafe ( unsafePerformIO ) -- for show instances

import Casadi.Core.Classes.PrintableObject
import Casadi.Internal.CToolsInstances ( )
import Casadi.Internal.FormatException ( formatException )
import Casadi.Internal.MarshalTypes ( StdVec, StdString) -- StdPair StdOstream'
import Casadi.Internal.Marshal ( Marshal(..), withMarshal )
import Casadi.Internal.WrapReturn ( WrapReturn(..) )
import Casadi.Core.Data
import Casadi.Core.Enums

instance Show LinearSolver where
  show = unsafePerformIO . printableObject_getDescription
-- direct wrapper
foreign import ccall unsafe "casadi__LinearSolver__CONSTRUCTOR__0" c_casadi__LinearSolver__CONSTRUCTOR__0
  :: Ptr (Ptr StdString) -> Ptr Sparsity' -> IO (Ptr LinearSolver')
casadi__LinearSolver__CONSTRUCTOR__0
  :: Sparsity -> IO LinearSolver
casadi__LinearSolver__CONSTRUCTOR__0 x0 =
  withMarshal x0 $ \x0' ->
  do
    errStrPtrP <- new nullPtr
    ret <- c_casadi__LinearSolver__CONSTRUCTOR__0 errStrPtrP x0'
    errStrPtr <- peek errStrPtrP
    free errStrPtrP
    if errStrPtr == nullPtr then wrapReturn ret else wrapReturn errStrPtr >>= (error . formatException)


-- classy wrapper
linearSolver__0 :: Sparsity -> IO LinearSolver
linearSolver__0 = casadi__LinearSolver__CONSTRUCTOR__0


-- direct wrapper
foreign import ccall unsafe "casadi__LinearSolver__CONSTRUCTOR__1" c_casadi__LinearSolver__CONSTRUCTOR__1
  :: Ptr (Ptr StdString) -> Ptr Sparsity' -> CInt -> IO (Ptr LinearSolver')
casadi__LinearSolver__CONSTRUCTOR__1
  :: Sparsity -> Int -> IO LinearSolver
casadi__LinearSolver__CONSTRUCTOR__1 x0 x1 =
  withMarshal x0 $ \x0' ->
  withMarshal x1 $ \x1' ->
  do
    errStrPtrP <- new nullPtr
    ret <- c_casadi__LinearSolver__CONSTRUCTOR__1 errStrPtrP x0' x1'
    errStrPtr <- peek errStrPtrP
    free errStrPtrP
    if errStrPtr == nullPtr then wrapReturn ret else wrapReturn errStrPtr >>= (error . formatException)


-- classy wrapper
linearSolver__1 :: Sparsity -> Int -> IO LinearSolver
linearSolver__1 = casadi__LinearSolver__CONSTRUCTOR__1


-- direct wrapper
foreign import ccall unsafe "casadi__LinearSolver__prepare" c_casadi__LinearSolver__prepare
  :: Ptr (Ptr StdString) -> Ptr LinearSolver' -> IO ()
casadi__LinearSolver__prepare
  :: LinearSolver -> IO ()
casadi__LinearSolver__prepare x0 =
  withMarshal x0 $ \x0' ->
  do
    errStrPtrP <- new nullPtr
    ret <- c_casadi__LinearSolver__prepare errStrPtrP x0'
    errStrPtr <- peek errStrPtrP
    free errStrPtrP
    if errStrPtr == nullPtr then wrapReturn ret else wrapReturn errStrPtr >>= (error . formatException)


-- classy wrapper
linearSolver_prepare :: LinearSolverClass a => a -> IO ()
linearSolver_prepare x = casadi__LinearSolver__prepare (castLinearSolver x)


-- direct wrapper
foreign import ccall unsafe "casadi__LinearSolver__prepared" c_casadi__LinearSolver__prepared
  :: Ptr (Ptr StdString) -> Ptr LinearSolver' -> IO CInt
casadi__LinearSolver__prepared
  :: LinearSolver -> IO Bool
casadi__LinearSolver__prepared x0 =
  withMarshal x0 $ \x0' ->
  do
    errStrPtrP <- new nullPtr
    ret <- c_casadi__LinearSolver__prepared errStrPtrP x0'
    errStrPtr <- peek errStrPtrP
    free errStrPtrP
    if errStrPtr == nullPtr then wrapReturn ret else wrapReturn errStrPtr >>= (error . formatException)


-- classy wrapper
linearSolver_prepared :: LinearSolverClass a => a -> IO Bool
linearSolver_prepared x = casadi__LinearSolver__prepared (castLinearSolver x)


-- direct wrapper
foreign import ccall unsafe "casadi__LinearSolver__solve__0" c_casadi__LinearSolver__solve__0
  :: Ptr (Ptr StdString) -> Ptr LinearSolver' -> Ptr MX' -> Ptr MX' -> IO (Ptr MX')
casadi__LinearSolver__solve__0
  :: LinearSolver -> MX -> MX -> IO MX
casadi__LinearSolver__solve__0 x0 x1 x2 =
  withMarshal x0 $ \x0' ->
  withMarshal x1 $ \x1' ->
  withMarshal x2 $ \x2' ->
  do
    errStrPtrP <- new nullPtr
    ret <- c_casadi__LinearSolver__solve__0 errStrPtrP x0' x1' x2'
    errStrPtr <- peek errStrPtrP
    free errStrPtrP
    if errStrPtr == nullPtr then wrapReturn ret else wrapReturn errStrPtr >>= (error . formatException)


-- classy wrapper
linearSolver_solve__0 :: LinearSolverClass a => a -> MX -> MX -> IO MX
linearSolver_solve__0 x = casadi__LinearSolver__solve__0 (castLinearSolver x)


-- direct wrapper
foreign import ccall unsafe "casadi__LinearSolver__solve__1" c_casadi__LinearSolver__solve__1
  :: Ptr (Ptr StdString) -> Ptr LinearSolver' -> Ptr MX' -> Ptr MX' -> CInt -> IO (Ptr MX')
casadi__LinearSolver__solve__1
  :: LinearSolver -> MX -> MX -> Bool -> IO MX
casadi__LinearSolver__solve__1 x0 x1 x2 x3 =
  withMarshal x0 $ \x0' ->
  withMarshal x1 $ \x1' ->
  withMarshal x2 $ \x2' ->
  withMarshal x3 $ \x3' ->
  do
    errStrPtrP <- new nullPtr
    ret <- c_casadi__LinearSolver__solve__1 errStrPtrP x0' x1' x2' x3'
    errStrPtr <- peek errStrPtrP
    free errStrPtrP
    if errStrPtr == nullPtr then wrapReturn ret else wrapReturn errStrPtr >>= (error . formatException)


-- classy wrapper
linearSolver_solve__1 :: LinearSolverClass a => a -> MX -> MX -> Bool -> IO MX
linearSolver_solve__1 x = casadi__LinearSolver__solve__1 (castLinearSolver x)


-- direct wrapper
foreign import ccall unsafe "casadi__LinearSolver__solve__2" c_casadi__LinearSolver__solve__2
  :: Ptr (Ptr StdString) -> Ptr LinearSolver' -> IO ()
casadi__LinearSolver__solve__2
  :: LinearSolver -> IO ()
casadi__LinearSolver__solve__2 x0 =
  withMarshal x0 $ \x0' ->
  do
    errStrPtrP <- new nullPtr
    ret <- c_casadi__LinearSolver__solve__2 errStrPtrP x0'
    errStrPtr <- peek errStrPtrP
    free errStrPtrP
    if errStrPtr == nullPtr then wrapReturn ret else wrapReturn errStrPtr >>= (error . formatException)


-- classy wrapper
linearSolver_solve__2 :: LinearSolverClass a => a -> IO ()
linearSolver_solve__2 x = casadi__LinearSolver__solve__2 (castLinearSolver x)


-- direct wrapper
foreign import ccall unsafe "casadi__LinearSolver__solve__3" c_casadi__LinearSolver__solve__3
  :: Ptr (Ptr StdString) -> Ptr LinearSolver' -> CInt -> IO ()
casadi__LinearSolver__solve__3
  :: LinearSolver -> Bool -> IO ()
casadi__LinearSolver__solve__3 x0 x1 =
  withMarshal x0 $ \x0' ->
  withMarshal x1 $ \x1' ->
  do
    errStrPtrP <- new nullPtr
    ret <- c_casadi__LinearSolver__solve__3 errStrPtrP x0' x1'
    errStrPtr <- peek errStrPtrP
    free errStrPtrP
    if errStrPtr == nullPtr then wrapReturn ret else wrapReturn errStrPtr >>= (error . formatException)


-- classy wrapper
linearSolver_solve__3 :: LinearSolverClass a => a -> Bool -> IO ()
linearSolver_solve__3 x = casadi__LinearSolver__solve__3 (castLinearSolver x)

