{-# OPTIONS_GHC -Wall #-}
{-# OPTIONS_GHC -fno-warn-unused-imports #-}
{-# OPTIONS_GHC -fno-warn-orphans #-}
{-# Language ForeignFunctionInterface #-}
{-# Language FlexibleInstances #-}
{-# Language MultiParamTypeClasses #-}

module Casadi.Core.Classes.NLPSolver
       (
         NLPSolver,
         NLPSolverClass(..),
         nlpSolver,
         nlpSolver_getReportConstraints,
         nlpSolver_gradF,
         nlpSolver_hessLag,
         nlpSolver_jacG,
         nlpSolver_joinFG,
         nlpSolver_nlp,
         nlpSolver_reportConstraints,
         nlpSolver_setQPOptions,
       ) where


import Prelude hiding ( Functor )

import Data.Vector ( Vector )
import Foreign.C.Types
import Foreign.Marshal ( new, free )
import Foreign.Storable ( peek )
import Foreign.Ptr ( Ptr, nullPtr )
import Foreign.ForeignPtr ( newForeignPtr )
import System.IO.Unsafe ( unsafePerformIO ) -- for show instances

import Casadi.Core.Classes.PrintableObject
import Casadi.Internal.CToolsInstances ( )
import Casadi.Internal.FormatException ( formatException )
import Casadi.Internal.MarshalTypes ( StdVec, StdString) -- StdPair StdOstream'
import Casadi.Internal.Marshal ( Marshal(..), withMarshal )
import Casadi.Internal.WrapReturn ( WrapReturn(..) )
import Casadi.Core.Data
import Casadi.Core.Enums

instance Show NLPSolver where
  show = unsafePerformIO . printableObject_getDescription
-- direct wrapper
foreign import ccall unsafe "casadi__NLPSolver__CONSTRUCTOR" c_casadi__NLPSolver__CONSTRUCTOR
  :: Ptr (Ptr StdString) -> IO (Ptr NLPSolver')
casadi__NLPSolver__CONSTRUCTOR
  :: IO NLPSolver
casadi__NLPSolver__CONSTRUCTOR  =

  do
    errStrPtrP <- new nullPtr
    ret <- c_casadi__NLPSolver__CONSTRUCTOR errStrPtrP 
    errStrPtr <- peek errStrPtrP
    free errStrPtrP
    if errStrPtr == nullPtr then wrapReturn ret else wrapReturn errStrPtr >>= (error . formatException)


-- classy wrapper
nlpSolver :: IO NLPSolver
nlpSolver = casadi__NLPSolver__CONSTRUCTOR


-- direct wrapper
foreign import ccall unsafe "casadi__NLPSolver__getReportConstraints" c_casadi__NLPSolver__getReportConstraints
  :: Ptr (Ptr StdString) -> Ptr NLPSolver' -> IO (Ptr StdString)
casadi__NLPSolver__getReportConstraints
  :: NLPSolver -> IO String
casadi__NLPSolver__getReportConstraints x0 =
  withMarshal x0 $ \x0' ->
  do
    errStrPtrP <- new nullPtr
    ret <- c_casadi__NLPSolver__getReportConstraints errStrPtrP x0'
    errStrPtr <- peek errStrPtrP
    free errStrPtrP
    if errStrPtr == nullPtr then wrapReturn ret else wrapReturn errStrPtr >>= (error . formatException)


-- classy wrapper
nlpSolver_getReportConstraints :: NLPSolverClass a => a -> IO String
nlpSolver_getReportConstraints x = casadi__NLPSolver__getReportConstraints (castNLPSolver x)


-- direct wrapper
foreign import ccall unsafe "casadi__NLPSolver__gradF" c_casadi__NLPSolver__gradF
  :: Ptr (Ptr StdString) -> Ptr NLPSolver' -> IO (Ptr Function')
casadi__NLPSolver__gradF
  :: NLPSolver -> IO Function
casadi__NLPSolver__gradF x0 =
  withMarshal x0 $ \x0' ->
  do
    errStrPtrP <- new nullPtr
    ret <- c_casadi__NLPSolver__gradF errStrPtrP x0'
    errStrPtr <- peek errStrPtrP
    free errStrPtrP
    if errStrPtr == nullPtr then wrapReturn ret else wrapReturn errStrPtr >>= (error . formatException)


-- classy wrapper
nlpSolver_gradF :: NLPSolverClass a => a -> IO Function
nlpSolver_gradF x = casadi__NLPSolver__gradF (castNLPSolver x)


-- direct wrapper
foreign import ccall unsafe "casadi__NLPSolver__hessLag" c_casadi__NLPSolver__hessLag
  :: Ptr (Ptr StdString) -> Ptr NLPSolver' -> IO (Ptr Function')
casadi__NLPSolver__hessLag
  :: NLPSolver -> IO Function
casadi__NLPSolver__hessLag x0 =
  withMarshal x0 $ \x0' ->
  do
    errStrPtrP <- new nullPtr
    ret <- c_casadi__NLPSolver__hessLag errStrPtrP x0'
    errStrPtr <- peek errStrPtrP
    free errStrPtrP
    if errStrPtr == nullPtr then wrapReturn ret else wrapReturn errStrPtr >>= (error . formatException)


-- classy wrapper
nlpSolver_hessLag :: NLPSolverClass a => a -> IO Function
nlpSolver_hessLag x = casadi__NLPSolver__hessLag (castNLPSolver x)


-- direct wrapper
foreign import ccall unsafe "casadi__NLPSolver__jacG" c_casadi__NLPSolver__jacG
  :: Ptr (Ptr StdString) -> Ptr NLPSolver' -> IO (Ptr Function')
casadi__NLPSolver__jacG
  :: NLPSolver -> IO Function
casadi__NLPSolver__jacG x0 =
  withMarshal x0 $ \x0' ->
  do
    errStrPtrP <- new nullPtr
    ret <- c_casadi__NLPSolver__jacG errStrPtrP x0'
    errStrPtr <- peek errStrPtrP
    free errStrPtrP
    if errStrPtr == nullPtr then wrapReturn ret else wrapReturn errStrPtr >>= (error . formatException)


-- classy wrapper
nlpSolver_jacG :: NLPSolverClass a => a -> IO Function
nlpSolver_jacG x = casadi__NLPSolver__jacG (castNLPSolver x)


-- direct wrapper
foreign import ccall unsafe "casadi__NLPSolver__joinFG" c_casadi__NLPSolver__joinFG
  :: Ptr (Ptr StdString) -> Ptr Function' -> Ptr Function' -> IO (Ptr Function')
casadi__NLPSolver__joinFG
  :: Function -> Function -> IO Function
casadi__NLPSolver__joinFG x0 x1 =
  withMarshal x0 $ \x0' ->
  withMarshal x1 $ \x1' ->
  do
    errStrPtrP <- new nullPtr
    ret <- c_casadi__NLPSolver__joinFG errStrPtrP x0' x1'
    errStrPtr <- peek errStrPtrP
    free errStrPtrP
    if errStrPtr == nullPtr then wrapReturn ret else wrapReturn errStrPtr >>= (error . formatException)


-- classy wrapper
nlpSolver_joinFG :: Function -> Function -> IO Function
nlpSolver_joinFG = casadi__NLPSolver__joinFG


-- direct wrapper
foreign import ccall unsafe "casadi__NLPSolver__nlp" c_casadi__NLPSolver__nlp
  :: Ptr (Ptr StdString) -> Ptr NLPSolver' -> IO (Ptr Function')
casadi__NLPSolver__nlp
  :: NLPSolver -> IO Function
casadi__NLPSolver__nlp x0 =
  withMarshal x0 $ \x0' ->
  do
    errStrPtrP <- new nullPtr
    ret <- c_casadi__NLPSolver__nlp errStrPtrP x0'
    errStrPtr <- peek errStrPtrP
    free errStrPtrP
    if errStrPtr == nullPtr then wrapReturn ret else wrapReturn errStrPtr >>= (error . formatException)


-- classy wrapper
nlpSolver_nlp :: NLPSolverClass a => a -> IO Function
nlpSolver_nlp x = casadi__NLPSolver__nlp (castNLPSolver x)


-- direct wrapper
foreign import ccall unsafe "casadi__NLPSolver__reportConstraints" c_casadi__NLPSolver__reportConstraints
  :: Ptr (Ptr StdString) -> Ptr NLPSolver' -> IO ()
casadi__NLPSolver__reportConstraints
  :: NLPSolver -> IO ()
casadi__NLPSolver__reportConstraints x0 =
  withMarshal x0 $ \x0' ->
  do
    errStrPtrP <- new nullPtr
    ret <- c_casadi__NLPSolver__reportConstraints errStrPtrP x0'
    errStrPtr <- peek errStrPtrP
    free errStrPtrP
    if errStrPtr == nullPtr then wrapReturn ret else wrapReturn errStrPtr >>= (error . formatException)


-- classy wrapper
nlpSolver_reportConstraints :: NLPSolverClass a => a -> IO ()
nlpSolver_reportConstraints x = casadi__NLPSolver__reportConstraints (castNLPSolver x)


-- direct wrapper
foreign import ccall unsafe "casadi__NLPSolver__setQPOptions" c_casadi__NLPSolver__setQPOptions
  :: Ptr (Ptr StdString) -> Ptr NLPSolver' -> IO ()
casadi__NLPSolver__setQPOptions
  :: NLPSolver -> IO ()
casadi__NLPSolver__setQPOptions x0 =
  withMarshal x0 $ \x0' ->
  do
    errStrPtrP <- new nullPtr
    ret <- c_casadi__NLPSolver__setQPOptions errStrPtrP x0'
    errStrPtr <- peek errStrPtrP
    free errStrPtrP
    if errStrPtr == nullPtr then wrapReturn ret else wrapReturn errStrPtr >>= (error . formatException)


-- classy wrapper
nlpSolver_setQPOptions :: NLPSolverClass a => a -> IO ()
nlpSolver_setQPOptions x = casadi__NLPSolver__setQPOptions (castNLPSolver x)

