{-# OPTIONS_GHC -Wall #-}
{-# OPTIONS_GHC -fno-warn-unused-imports #-}
{-# OPTIONS_GHC -fno-warn-orphans #-}
{-# Language ForeignFunctionInterface #-}
{-# Language FlexibleInstances #-}
{-# Language MultiParamTypeClasses #-}

module Casadi.Core.Classes.OCPSolver
       (
         OCPSolver,
         OCPSolverClass(..),
         ocpSolver,
         ocpSolver_getCfcn,
         ocpSolver_getFfcn,
         ocpSolver_getMfcn,
         ocpSolver_getRfcn,
       ) where


import Prelude hiding ( Functor )

import Data.Vector ( Vector )
import Foreign.C.Types
import Foreign.Marshal ( new, free )
import Foreign.Storable ( peek )
import Foreign.Ptr ( Ptr, nullPtr )
import Foreign.ForeignPtr ( newForeignPtr )
import System.IO.Unsafe ( unsafePerformIO ) -- for show instances

import Casadi.Core.Classes.PrintableObject
import Casadi.Internal.CToolsInstances ( )
import Casadi.Internal.FormatException ( formatException )
import Casadi.Internal.MarshalTypes ( StdVec, StdString) -- StdPair StdOstream'
import Casadi.Internal.Marshal ( Marshal(..), withMarshal )
import Casadi.Internal.WrapReturn ( WrapReturn(..) )
import Casadi.Core.Data
import Casadi.Core.Enums

instance Show OCPSolver where
  show = unsafePerformIO . printableObject_getDescription
-- direct wrapper
foreign import ccall unsafe "casadi__OCPSolver__CONSTRUCTOR" c_casadi__OCPSolver__CONSTRUCTOR
  :: Ptr (Ptr StdString) -> IO (Ptr OCPSolver')
casadi__OCPSolver__CONSTRUCTOR
  :: IO OCPSolver
casadi__OCPSolver__CONSTRUCTOR  =

  do
    errStrPtrP <- new nullPtr
    ret <- c_casadi__OCPSolver__CONSTRUCTOR errStrPtrP 
    errStrPtr <- peek errStrPtrP
    free errStrPtrP
    if errStrPtr == nullPtr then wrapReturn ret else wrapReturn errStrPtr >>= (error . formatException)


-- classy wrapper
ocpSolver :: IO OCPSolver
ocpSolver = casadi__OCPSolver__CONSTRUCTOR


-- direct wrapper
foreign import ccall unsafe "casadi__OCPSolver__getCfcn" c_casadi__OCPSolver__getCfcn
  :: Ptr (Ptr StdString) -> Ptr OCPSolver' -> IO (Ptr Function')
casadi__OCPSolver__getCfcn
  :: OCPSolver -> IO Function
casadi__OCPSolver__getCfcn x0 =
  withMarshal x0 $ \x0' ->
  do
    errStrPtrP <- new nullPtr
    ret <- c_casadi__OCPSolver__getCfcn errStrPtrP x0'
    errStrPtr <- peek errStrPtrP
    free errStrPtrP
    if errStrPtr == nullPtr then wrapReturn ret else wrapReturn errStrPtr >>= (error . formatException)


-- classy wrapper
ocpSolver_getCfcn :: OCPSolverClass a => a -> IO Function
ocpSolver_getCfcn x = casadi__OCPSolver__getCfcn (castOCPSolver x)


-- direct wrapper
foreign import ccall unsafe "casadi__OCPSolver__getFfcn" c_casadi__OCPSolver__getFfcn
  :: Ptr (Ptr StdString) -> Ptr OCPSolver' -> IO (Ptr Function')
casadi__OCPSolver__getFfcn
  :: OCPSolver -> IO Function
casadi__OCPSolver__getFfcn x0 =
  withMarshal x0 $ \x0' ->
  do
    errStrPtrP <- new nullPtr
    ret <- c_casadi__OCPSolver__getFfcn errStrPtrP x0'
    errStrPtr <- peek errStrPtrP
    free errStrPtrP
    if errStrPtr == nullPtr then wrapReturn ret else wrapReturn errStrPtr >>= (error . formatException)


-- classy wrapper
ocpSolver_getFfcn :: OCPSolverClass a => a -> IO Function
ocpSolver_getFfcn x = casadi__OCPSolver__getFfcn (castOCPSolver x)


-- direct wrapper
foreign import ccall unsafe "casadi__OCPSolver__getMfcn" c_casadi__OCPSolver__getMfcn
  :: Ptr (Ptr StdString) -> Ptr OCPSolver' -> IO (Ptr Function')
casadi__OCPSolver__getMfcn
  :: OCPSolver -> IO Function
casadi__OCPSolver__getMfcn x0 =
  withMarshal x0 $ \x0' ->
  do
    errStrPtrP <- new nullPtr
    ret <- c_casadi__OCPSolver__getMfcn errStrPtrP x0'
    errStrPtr <- peek errStrPtrP
    free errStrPtrP
    if errStrPtr == nullPtr then wrapReturn ret else wrapReturn errStrPtr >>= (error . formatException)


-- classy wrapper
ocpSolver_getMfcn :: OCPSolverClass a => a -> IO Function
ocpSolver_getMfcn x = casadi__OCPSolver__getMfcn (castOCPSolver x)


-- direct wrapper
foreign import ccall unsafe "casadi__OCPSolver__getRfcn" c_casadi__OCPSolver__getRfcn
  :: Ptr (Ptr StdString) -> Ptr OCPSolver' -> IO (Ptr Function')
casadi__OCPSolver__getRfcn
  :: OCPSolver -> IO Function
casadi__OCPSolver__getRfcn x0 =
  withMarshal x0 $ \x0' ->
  do
    errStrPtrP <- new nullPtr
    ret <- c_casadi__OCPSolver__getRfcn errStrPtrP x0'
    errStrPtr <- peek errStrPtrP
    free errStrPtrP
    if errStrPtr == nullPtr then wrapReturn ret else wrapReturn errStrPtr >>= (error . formatException)


-- classy wrapper
ocpSolver_getRfcn :: OCPSolverClass a => a -> IO Function
ocpSolver_getRfcn x = casadi__OCPSolver__getRfcn (castOCPSolver x)

