{-# OPTIONS_GHC -Wall #-}
{-# OPTIONS_GHC -fno-warn-unused-imports #-}
{-# OPTIONS_GHC -fno-warn-orphans #-}
{-# Language ForeignFunctionInterface #-}
{-# Language FlexibleInstances #-}
{-# Language MultiParamTypeClasses #-}

module Casadi.Core.Classes.QCQPSolver
       (
         QCQPSolver,
         QCQPSolverClass(..),
         qcqpSolver,
         qcqpSolver_setQPOptions,
       ) where


import Prelude hiding ( Functor )

import Data.Vector ( Vector )
import Foreign.C.Types
import Foreign.Marshal ( new, free )
import Foreign.Storable ( peek )
import Foreign.Ptr ( Ptr, nullPtr )
import Foreign.ForeignPtr ( newForeignPtr )
import System.IO.Unsafe ( unsafePerformIO ) -- for show instances

import Casadi.Core.Classes.PrintableObject
import Casadi.Internal.CToolsInstances ( )
import Casadi.Internal.FormatException ( formatException )
import Casadi.Internal.MarshalTypes ( StdVec, StdString) -- StdPair StdOstream'
import Casadi.Internal.Marshal ( Marshal(..), withMarshal )
import Casadi.Internal.WrapReturn ( WrapReturn(..) )
import Casadi.Core.Data
import Casadi.Core.Enums

instance Show QCQPSolver where
  show = unsafePerformIO . printableObject_getDescription
-- direct wrapper
foreign import ccall unsafe "casadi__QCQPSolver__CONSTRUCTOR" c_casadi__QCQPSolver__CONSTRUCTOR
  :: Ptr (Ptr StdString) -> IO (Ptr QCQPSolver')
casadi__QCQPSolver__CONSTRUCTOR
  :: IO QCQPSolver
casadi__QCQPSolver__CONSTRUCTOR  =

  do
    errStrPtrP <- new nullPtr
    ret <- c_casadi__QCQPSolver__CONSTRUCTOR errStrPtrP 
    errStrPtr <- peek errStrPtrP
    free errStrPtrP
    if errStrPtr == nullPtr then wrapReturn ret else wrapReturn errStrPtr >>= (error . formatException)


-- classy wrapper
qcqpSolver :: IO QCQPSolver
qcqpSolver = casadi__QCQPSolver__CONSTRUCTOR


-- direct wrapper
foreign import ccall unsafe "casadi__QCQPSolver__setQPOptions" c_casadi__QCQPSolver__setQPOptions
  :: Ptr (Ptr StdString) -> Ptr QCQPSolver' -> IO ()
casadi__QCQPSolver__setQPOptions
  :: QCQPSolver -> IO ()
casadi__QCQPSolver__setQPOptions x0 =
  withMarshal x0 $ \x0' ->
  do
    errStrPtrP <- new nullPtr
    ret <- c_casadi__QCQPSolver__setQPOptions errStrPtrP x0'
    errStrPtr <- peek errStrPtrP
    free errStrPtrP
    if errStrPtr == nullPtr then wrapReturn ret else wrapReturn errStrPtr >>= (error . formatException)


-- classy wrapper
qcqpSolver_setQPOptions :: QCQPSolverClass a => a -> IO ()
qcqpSolver_setQPOptions x = casadi__QCQPSolver__setQPOptions (castQCQPSolver x)

