{-# OPTIONS_GHC -Wall #-}
{-# OPTIONS_GHC -fno-warn-unused-imports #-}
{-# OPTIONS_GHC -fno-warn-orphans #-}
{-# Language ForeignFunctionInterface #-}
{-# Language FlexibleInstances #-}
{-# Language MultiParamTypeClasses #-}

module Casadi.Core.Classes.SDPSolver
       (
         SDPSolver,
         SDPSolverClass(..),
         sdpSolver,
         sdpSolver_setSOCPOptions,
       ) where


import Prelude hiding ( Functor )

import Data.Vector ( Vector )
import Foreign.C.Types
import Foreign.Marshal ( new, free )
import Foreign.Storable ( peek )
import Foreign.Ptr ( Ptr, nullPtr )
import Foreign.ForeignPtr ( newForeignPtr )
import System.IO.Unsafe ( unsafePerformIO ) -- for show instances

import Casadi.Core.Classes.PrintableObject
import Casadi.Internal.CToolsInstances ( )
import Casadi.Internal.FormatException ( formatException )
import Casadi.Internal.MarshalTypes ( StdVec, StdString) -- StdPair StdOstream'
import Casadi.Internal.Marshal ( Marshal(..), withMarshal )
import Casadi.Internal.WrapReturn ( WrapReturn(..) )
import Casadi.Core.Data
import Casadi.Core.Enums

instance Show SDPSolver where
  show = unsafePerformIO . printableObject_getDescription
-- direct wrapper
foreign import ccall unsafe "casadi__SDPSolver__CONSTRUCTOR" c_casadi__SDPSolver__CONSTRUCTOR
  :: Ptr (Ptr StdString) -> IO (Ptr SDPSolver')
casadi__SDPSolver__CONSTRUCTOR
  :: IO SDPSolver
casadi__SDPSolver__CONSTRUCTOR  =

  do
    errStrPtrP <- new nullPtr
    ret <- c_casadi__SDPSolver__CONSTRUCTOR errStrPtrP 
    errStrPtr <- peek errStrPtrP
    free errStrPtrP
    if errStrPtr == nullPtr then wrapReturn ret else wrapReturn errStrPtr >>= (error . formatException)


-- classy wrapper
sdpSolver :: IO SDPSolver
sdpSolver = casadi__SDPSolver__CONSTRUCTOR


-- direct wrapper
foreign import ccall unsafe "casadi__SDPSolver__setSOCPOptions" c_casadi__SDPSolver__setSOCPOptions
  :: Ptr (Ptr StdString) -> Ptr SDPSolver' -> IO ()
casadi__SDPSolver__setSOCPOptions
  :: SDPSolver -> IO ()
casadi__SDPSolver__setSOCPOptions x0 =
  withMarshal x0 $ \x0' ->
  do
    errStrPtrP <- new nullPtr
    ret <- c_casadi__SDPSolver__setSOCPOptions errStrPtrP x0'
    errStrPtr <- peek errStrPtrP
    free errStrPtrP
    if errStrPtr == nullPtr then wrapReturn ret else wrapReturn errStrPtr >>= (error . formatException)


-- classy wrapper
sdpSolver_setSOCPOptions :: SDPSolverClass a => a -> IO ()
sdpSolver_setSOCPOptions x = casadi__SDPSolver__setSOCPOptions (castSDPSolver x)

