{-# OPTIONS_GHC -Wall #-}
{-# OPTIONS_GHC -fno-warn-unused-imports #-}
{-# OPTIONS_GHC -fno-warn-orphans #-}
{-# Language ForeignFunctionInterface #-}
{-# Language FlexibleInstances #-}
{-# Language MultiParamTypeClasses #-}

module Casadi.Core.Classes.SDQPSolver
       (
         SDQPSolver,
         SDQPSolverClass(..),
         sdqpSolver,
         sdqpSolver_setSOCQPOptions,
       ) where


import Prelude hiding ( Functor )

import Data.Vector ( Vector )
import Foreign.C.Types
import Foreign.Marshal ( new, free )
import Foreign.Storable ( peek )
import Foreign.Ptr ( Ptr, nullPtr )
import Foreign.ForeignPtr ( newForeignPtr )
import System.IO.Unsafe ( unsafePerformIO ) -- for show instances

import Casadi.Core.Classes.PrintableObject
import Casadi.Internal.CToolsInstances ( )
import Casadi.Internal.FormatException ( formatException )
import Casadi.Internal.MarshalTypes ( StdVec, StdString) -- StdPair StdOstream'
import Casadi.Internal.Marshal ( Marshal(..), withMarshal )
import Casadi.Internal.WrapReturn ( WrapReturn(..) )
import Casadi.Core.Data
import Casadi.Core.Enums

instance Show SDQPSolver where
  show = unsafePerformIO . printableObject_getDescription
-- direct wrapper
foreign import ccall unsafe "casadi__SDQPSolver__CONSTRUCTOR" c_casadi__SDQPSolver__CONSTRUCTOR
  :: Ptr (Ptr StdString) -> IO (Ptr SDQPSolver')
casadi__SDQPSolver__CONSTRUCTOR
  :: IO SDQPSolver
casadi__SDQPSolver__CONSTRUCTOR  =

  do
    errStrPtrP <- new nullPtr
    ret <- c_casadi__SDQPSolver__CONSTRUCTOR errStrPtrP 
    errStrPtr <- peek errStrPtrP
    free errStrPtrP
    if errStrPtr == nullPtr then wrapReturn ret else wrapReturn errStrPtr >>= (error . formatException)


-- classy wrapper
sdqpSolver :: IO SDQPSolver
sdqpSolver = casadi__SDQPSolver__CONSTRUCTOR


-- direct wrapper
foreign import ccall unsafe "casadi__SDQPSolver__setSOCQPOptions" c_casadi__SDQPSolver__setSOCQPOptions
  :: Ptr (Ptr StdString) -> Ptr SDQPSolver' -> IO ()
casadi__SDQPSolver__setSOCQPOptions
  :: SDQPSolver -> IO ()
casadi__SDQPSolver__setSOCQPOptions x0 =
  withMarshal x0 $ \x0' ->
  do
    errStrPtrP <- new nullPtr
    ret <- c_casadi__SDQPSolver__setSOCQPOptions errStrPtrP x0'
    errStrPtr <- peek errStrPtrP
    free errStrPtrP
    if errStrPtr == nullPtr then wrapReturn ret else wrapReturn errStrPtr >>= (error . formatException)


-- classy wrapper
sdqpSolver_setSOCQPOptions :: SDQPSolverClass a => a -> IO ()
sdqpSolver_setSOCQPOptions x = casadi__SDQPSolver__setSOCQPOptions (castSDQPSolver x)

